/*
 * Copyright (c) 2011 Kristaps Dzonsons <kristaps@bsd.lv>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "extern.h"

static	const char * const greeks[GREEK__MAX] = {
	"alpha",
	"theta",
	"omicron",
	"tau",
	"beta",
	"pi",
	"upsilon",
	"gamma",
	"phi",
	"delta",
	"kappa",
	"rho",
	"epsilon",
	"lambda",
	"chi",
	"mu",
	"sigma",
	"psi",
	"zeta",
	"nu",
	"omega",
	"eta",
	"xi",
	"Gamma",
	"Delta",
	"Theta",
	"Pi",
	"Phi",
	"Lambda",
	"Sigma",
	"Psi",
	"Xi",
	"Upsilon",
	"Omega",
};

void
symtab_free(struct symtab *sp)
{
	struct sym	*sym, *symp;

	sym = sp->syms;
	while (sym) {
		symp = sym->next;
		free(sym->name);
		free(sym);
		sym = symp;
	}
}

int
symtab_has(struct symtab *sp, const char *p, size_t sz)
{
	struct sym	*sym;

	for (sym = sp->syms; sym; sym = sym->next)
		if (0 == strncmp(sym->name, p, sz) && 
				'\0' == sym->name[sz])
			return(1);

	return(0);
}

struct sym *
symtab_add(struct symtab *sp, const char *p, size_t sz)
{
	struct sym	*sym;
	size_t		 i;

	/* Search through the symbols for a match. */

	for (sym = sp->syms; sym; sym = sym->next)
		if (0 == strncmp(sym->name, p, sz) && 
				'\0' == sym->name[sz])
			return(sym);

	/* If no symbol's been found, make a new one. */

	if (NULL == (sym = calloc(1, sizeof(struct sym)))) {
		perror(NULL);
		exit(EXIT_FAILURE);
	}

	if (NULL == (sym->name = malloc(sz + 1))) {
		perror(NULL);
		exit(EXIT_FAILURE);
	}

	memcpy(sym->name, p, sz);
	sym->name[sz] = '\0';

	for (i = 0; i < GREEK__MAX; i++)
		if (0 == strcmp(greeks[i], sym->name))
			break;

	sym->greek = (enum greek)i;

	if (NULL != sp->syms)
		sym->next = sp->syms;

	sp->syms = sym;
	return(sym);
}
